import anime from 'animejs'
import Circle from './_circle'
import Point from './_point'
import debounce from './_debounce'

export default class {
  constructor(canvas, nodeList) {
    this.ratio = window.devicePixelRatio;
    this.nodeList = nodeList;
    this.canvas = canvas;
    this.ctx = this.canvas.getContext('2d');
    this.resolution = {
      x: window.innerWidth * this.ratio,
      y: window.innerHeight * this.ratio
    };
    this.points = [];
    this._render = this._render.bind(this);
    this.cursor = new Point(this);
    this._init();
  }

  _init() {
    this.canvas.width = this.resolution.x;
    this.canvas.height = this.resolution.y;

    for(let x = 0; x < this.resolution.x; x += this.resolution.x / 16) {
      for(let y = 0; y < this.resolution.y; y += this.resolution.y / 16) {
        const px = x + Math.random() * this.resolution.x / 16;
        const py = y + Math.random() * this.resolution.y / 16;
        this.points.push({ x: px, originX: px, y: py, originY: py })
      }
    }

    // ポイントの初期化
    this.points.forEach(p1 => {
      let closest = [];
      this.points.forEach(p2 => {
        if(p1 === p2) return;
        let placed = false;
        [...Array(4).keys()].forEach(i => {
          if(placed === true) return;
          if(closest[i] == undefined) {
            closest[i] = p2;
            placed = true;
          }
        });
        [...Array(4).keys()].forEach(i => {
          if(placed === true) return;
          if(this._distance(p1, p2) < this._distance(p1, closest[i])) {
            closest[i] = p2;
            placed = true;
          }
        });
        p1.closest = closest;
      });
    });

    this.points.map(x => {
      x.circle = new Circle(x, 3 + Math.random() * 3, 'rgba(139, 218, 234, 1)', this.ctx);
      x.circle._move();
      return x;
    });

    // kvアニメーション
    anime({
      targets: this.nodeList,
      rotateX: 0,
      rotateY: 0,
      rotateZ: 0,
      translateX: 0,
      translateY: 0,
      translateZ: 0,
      opacity: 1,
      duration: 1000,
      delay: 400,
      easing: 'easeInOutQuart',
      complete: () => {
        this.nodeList.forEach(x => x.removeAttribute('style'));
      }
    });

    // レンダリング
    this._enabled();
    this._render();
  }

  _enabled() {
    window.addEventListener('resize', debounce(this._resize.bind(this), 300), false);
  }

  _resize() {
    // 変更後のサイズ
    const resolution = {
      x: window.innerWidth * this.ratio,
      y: window.innerHeight * this.ratio
    };
    // 比率
    const ratio = {
      x: resolution.x / this.resolution.x,
      y: resolution.y / this.resolution.y
    };

    this.resolution.x = resolution.x;
    this.resolution.y = resolution.y;
    this.canvas.width = this.resolution.x;
    this.canvas.height = this.resolution.y;

    // 比率から位置を修正
    this.points.forEach(o => {
      o.originX = o.originX * ratio.x;
      o.originY = o.originY * ratio.y;
    });
  }

  _render() {
    this.ctx.clearRect(0,0, this.resolution.x, this.resolution.y);
    this.points.forEach(x => {
      this._draw(x);
      x.circle._draw();
    });

    // Cursor
    this.cursor._render();

    // roop
    requestAnimationFrame(this._render);
  }

  _draw(o) {
    o.closest.forEach(i => {
      this.ctx.beginPath();
      this.ctx.moveTo(o.x, o.y);
      this.ctx.lineTo(i.x, i.y);
      this.ctx.strokeStyle = 'rgba(139, 218, 234, 0.3)';
      this.ctx.stroke();
    });
  }

  _distance(p1, p2) {
    return Math.pow(p1.x - p2.x, 2) + Math.pow(p1.y - p2.y, 2);
  }
}
