
class Item {
  constructor(o) {
    this.elm = o.elm;
    this.k = o.k;
  }

  updated({x, y}) {
    const kx = Math.round(x * 10 * this.k) / 10;
    const ky = Math.round(y * 10 * this.k) / 10;
    [...this.elm].forEach(node => {
      node.style.transform = `translateX(${kx}px) translateY(${ky}px)`;
    });
  }
}

export default class {
  constructor(self) {
    this.nodeList = [];
    this.modules = self;
    this.diff = { x: 0, y: 0 };
    this.position = { x: 0, y: 0 };
    this.default = { x: 0, y: 0 };
    this._init();
    this._enabled();
  }

  _init() {
    const node01 = document.querySelectorAll('.js-title');
    const node02 = document.querySelectorAll('#canvas');
    this.nodeList.push(new Item({ elm: node01, k: 0.8 }));
    this.nodeList.push(new Item({ elm: node02, k: 0.3 }));
  }

  _enabled() {
    // deviceorientation
    window.addEventListener('deviceorientation', this._deviceorientation.bind(this), false);

    // mousemove
    document.addEventListener('mousemove', this._mousemove.bind(this), false);
  }

  _render() {
    this.position.x += (this.diff.x - this.position.x) * 0.03;
    this.position.y += (this.diff.y - this.position.y) * 0.03;
    this.nodeList.forEach(x => {
      x.updated(this.position);
    });
  }

  _mousemove(e) {
    const {x, y} = this.modules.resolution;
    const {pageX, pageY} = e;
    const position = {
      x: -((pageX / x * 2) - 1) * 90,
      y: -((pageY / y * 2) - 1) * 90
    }
    this.diff = position;
  }

  _deviceorientation(e) {
    let position = {
      x: e.gamma,
      y: Math.max(-90, Math.min(e.beta, 90))
    }
    // デフォルト値として保存
    if(this.default.x === 0 && this.default.y === 0) {
      this.default.x = position.x;
      this.default.y = position.y;
    }
    position.x = position.x - this.default.x;
    position.y = position.y - this.default.y;
    this.diff = position;
  }
}
