const path = require('path');
const env = process.env.NODE_ENV;
const webpack = require('webpack');
const WebpackBuildNotifierPlugin = require('webpack-build-notifier');
const merge = require('webpack-merge');
const HtmlWebpackPlugin = require('html-webpack-plugin');

const mode = {
  dev: require('./webpack.dev.js'),
  prod: require('./webpack.prod.js')
};

// 開発 or 本番
const DEBUG = 0 ? true : false;

const config = {
  entry: {
    bundle: './src/js/app.js'
  },
  output: {
    filename: '[name].js'
  },
  resolve: {
    modules: [path.join(__dirname, 'node_modules')],
    extensions: ['.webpack.js', '.web.js', '.js'],
    alias: { '@': path.join(__dirname, 'src/js') }
  },
  plugins: [
    new webpack.ProvidePlugin({
      $: 'jquery',
      jQuery: 'jquery',
      anime: 'animejs'
    }),
    new webpack.DefinePlugin({
      'process.env.NODE_ENV' : JSON.stringify(env)
    }),
    new WebpackBuildNotifierPlugin({
      sound: false,
    }),
    new webpack.optimize.OccurrenceOrderPlugin(),
    new HtmlWebpackPlugin({
      inject: 'head',
      template: './src/index.html'
    })
  ],
  optimization: {
    splitChunks: {
      cacheGroups: {
        vendor: {
          test: /(node_modules|src\/js\/lib)/,
          name: 'vendor',
          chunks: 'initial',
          enforce: true
        }
      }
    }
  },
  module: {
    rules: [
      {
        test: /\.js$/,
        exclude: /(node_modules)/,
        use: [
          {
            loader: "babel-loader",
            options: {
              presets: [
                ["@babel/preset-env", {
                  useBuiltIns: "entry",
                  corejs: 3
                }]
              ]
            }
          }
        ]
      },
      {
        test: /\.scss$/,
        use: [
          "style-loader",
          {
            loader: "css-loader",
            options: {
              url: false,
              sourceMap: DEBUG,
              importLoaders: 2
            }
          },
          {
            loader: "sass-loader",
            options: {
              sourceMap: DEBUG
            }
          },
          {
            loader: "postcss-loader",
            options: {
              sourceMap: true,
              plugins: [
                require("autoprefixer")({ grid: true })
              ]
            }
          },
        ]
      },
      {
        test: /\.css$/,
        use: [
          "style-loader",
          {
            loader: "css-loader",
            options: {
              url: false,
              sourceMap: DEBUG
            }
          }
        ]
      },
      {
        test: /\.html$/i,
        loader: 'html-loader',
      },
    ]
  }
};

module.exports = DEBUG
  ? merge(config, mode.dev)
  : merge(config, mode.prod);
